#ifndef CONFIG_REBOOT_H
#define CONFIG_REBOOT_H

/** @file
 *
 * Reboot API configuration
 *
 */

FILE_LICENCE ( GPL2_OR_LATER );

#include <config/defaults.h>

#include <config/local/reboot.h>

#endif /* CONFIG_REBOOT_H */
