#ifndef ETHERBOOT_ENDIAN_H
#define ETHERBOOT_ENDIAN_H

FILE_LICENCE ( GPL2_OR_LATER );

/* Definitions for byte order, according to significance of bytes,
   from low addresses to high addresses.  The value is what you get by
   putting '4' in the most significant byte, '3' in the second most
   significant byte, '2' in the second least significant byte, and '1'
   in the least significant byte, and then writing down one digit for
   each byte, starting with the byte at the lowest address at the left,
   and proceeding to the byte with the highest address at the right.  */

#define __LITTLE_ENDIAN 1234
#define __BIG_ENDIAN    4321
#define __PDP_ENDIAN    3412

#include "bits/endian.h"


#endif /* ETHERBOOT_ENDIAN_H */
