#ifndef AXTLS_CONFIG_H
#define AXTLS_CONFIG_H

/**
 * @file config.h
 *
 * Trick the axtls code into building within our build environment.
 */

#define CONFIG_SSL_ENABLE_CLIENT 1
#define CONFIG_BIGINT_CLASSICAL 1

#endif
