#ifndef _BITS_TIMER_H
#define _BITS_TIMER_H

/** @file
 *
 * i386-specific timer API implementations
 *
 */

FILE_LICENCE ( GPL2_OR_LATER );

#include <ipxe/bios_timer.h>
#include <ipxe/rdtsc_timer.h>

#endif /* _BITS_TIMER_H */
