#ifndef CONFIG_ENTROPY_H
#define CONFIG_ENTROPY_H

/** @file
 *
 * Entropy API configuration
 *
 */

FILE_LICENCE ( GPL2_OR_LATER );

#include <config/defaults.h>

#include <config/local/entropy.h>

#endif /* CONFIG_ENTROPY_H */
