#ifndef _BITS_SANBOOT_H
#define _BITS_SANBOOT_H

/** @file
 *
 * i386-specific sanboot API implementations
 *
 */

FILE_LICENCE ( GPL2_OR_LATER );

#include <ipxe/bios_sanboot.h>

#endif /* _BITS_SANBOOT_H */
