#ifndef _IPXE_SCRIPT_H
#define _IPXE_SCRIPT_H

/** @file
 *
 * iPXE scripts
 *
 */

FILE_LICENCE ( GPL2_OR_LATER );

#include <ipxe/image.h>

extern struct image_type script_image_type __image_type ( PROBE_NORMAL );

#endif /* _IPXE_SCRIPT_H */
