#ifndef CONFIG_UMALLOC_H
#define CONFIG_UMALLOC_H

/** @file
 *
 * User memory allocation API configuration
 *
 */

FILE_LICENCE ( GPL2_OR_LATER );

#include <config/defaults.h>

#include <config/local/umalloc.h>

#endif /* CONFIG_UMALLOC_H */
